@@ -16,19 +16,18 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ev-document-misc.h 2506 2007-06-17 15:05:11Z carlosgc $
+ *  $Id: ev-document-misc.h 3240 2008-10-27 12:17:08Z chpe $
  */
 
 #ifndef EV_DOCUMENT_MISC_H
 #define EV_DOCUMENT_MISC_H
 
-
-#include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gtk/gtkstyle.h>
 #include <cairo.h>
 
-G_BEGIN_DECLS
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gtk/gtk.h>
 
+G_BEGIN_DECLS
 
 GdkPixbuf *ev_document_misc_get_thumbnail_frame  (int           width,
 						  int           height,
